@@ -733,15 +733,19 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
 
         device = g_hash_table_lookup (manager_xi2->devices_by_id,
                                       GINT_TO_POINTER (xev->deviceid));
+        source_device = g_hash_table_lookup (manager_xi2->devices_by_id,
+                                             GINT_TO_POINTER (xev->sourceid));
         if (device)
           {
             _clutter_input_device_reset_axes (device);
-            _clutter_input_device_reset_scroll_info (device);
             translate_device_classes (backend_x11->xdpy,
                                       device,
                                       xev->classes,
                                       xev->num_classes);
           }
+
+        if (source_device)
+          _clutter_input_device_reset_scroll_info (device);
       }
       retval = CLUTTER_TRANSLATE_REMOVE;
       break;
@@ -832,6 +836,17 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
       {
         XIDeviceEvent *xev = (XIDeviceEvent *) xi_event;
 
+        source_device = g_hash_table_lookup (manager_xi2->devices_by_id,
+                                             GINT_TO_POINTER (xev->sourceid));
+        device = g_hash_table_lookup (manager_xi2->devices_by_id,
+                                      GINT_TO_POINTER (xev->deviceid));
+
+        /* Set the stage for core events coming out of nowhere (see bug #684509) */
+        if (clutter_input_device_get_device_mode (device) == CLUTTER_INPUT_MODE_MASTER &&
+            clutter_input_device_get_pointer_stage (device) == NULL &&
+            stage != NULL)
+          _clutter_input_device_set_stage (device, stage);
+
         switch (xev->detail)
           {
           case 4:
@@ -862,12 +877,7 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
               _clutter_input_device_xi2_translate_state (&xev->mods,
                                                          &xev->buttons);
 
-            source_device = g_hash_table_lookup (manager_xi2->devices_by_id,
-                                                 GINT_TO_POINTER (xev->sourceid));
             clutter_event_set_source_device (event, source_device);
-
-            device = g_hash_table_lookup (manager_xi2->devices_by_id,
-                                          GINT_TO_POINTER (xev->deviceid));
             clutter_event_set_device (event, device);
 
             event->scroll.axes = translate_axes (event->scroll.device,
@@ -877,11 +887,12 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
                                                  &xev->valuators);
 
             CLUTTER_NOTE (EVENT,
-                          "scroll: win:0x%x, device:%s, time:%d "
+                          "scroll: win:0x%x, device:%d '%s', time:%d "
                           "(direction:%s, "
                           "x:%.2f, y:%.2f, "
                           "emulated:%s)",
                           (unsigned int) stage_x11->xwin,
+                          device->id,
                           device->device_name,
                           event->any.time,
                           event->scroll.direction == CLUTTER_SCROLL_UP ? "up" :
@@ -914,12 +925,7 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
               _clutter_input_device_xi2_translate_state (&xev->mods,
                                                          &xev->buttons);
 
-            source_device = g_hash_table_lookup (manager_xi2->devices_by_id,
-                                                 GINT_TO_POINTER (xev->sourceid));
             clutter_event_set_source_device (event, source_device);
-
-            device = g_hash_table_lookup (manager_xi2->devices_by_id,
-                                          GINT_TO_POINTER (xev->deviceid));
             clutter_event_set_device (event, device);
 
             event->button.axes = translate_axes (event->button.device,
@@ -929,7 +935,7 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
                                                  &xev->valuators);
 
             CLUTTER_NOTE (EVENT,
-                          "%s: win:0x%x, device:%s, time:%d "
+                          "%s: win:0x%x, device:%d '%s', time:%d "
                           "(button:%d, "
                           "x:%.2f, y:%.2f, "
                           "axes:%s, "
@@ -938,6 +944,7 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
                             ? "button press  "
                             : "button release",
                           (unsigned int) stage_x11->xwin,
+                          device->id,
                           device->device_name,
                           event->any.time,
                           event->button.button,
@@ -975,6 +982,14 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
 
         source_device = g_hash_table_lookup (manager_xi2->devices_by_id,
                                              GINT_TO_POINTER (xev->sourceid));
+        device = g_hash_table_lookup (manager_xi2->devices_by_id,
+                                      GINT_TO_POINTER (xev->deviceid));
+
+        /* Set the stage for core events coming out of nowhere (see bug #684509) */
+        if (clutter_input_device_get_device_mode (device) == CLUTTER_INPUT_MODE_MASTER &&
+            clutter_input_device_get_pointer_stage (device) == NULL &&
+            stage != NULL)
+          _clutter_input_device_set_stage (device, stage);
 
         if (scroll_valuators_changed (source_device,
                                       &xev->valuators,
@@ -993,14 +1008,12 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
 
             clutter_event_set_scroll_delta (event, delta_x, delta_y);
             clutter_event_set_source_device (event, source_device);
-
-            device = g_hash_table_lookup (manager_xi2->devices_by_id,
-                                          GINT_TO_POINTER (xev->deviceid));
             clutter_event_set_device (event, device);
 
             CLUTTER_NOTE (EVENT,
-                          "smooth scroll: win:0x%x device:%s (x:%.2f, y:%.2f, delta:%f, %f)",
+                          "smooth scroll: win:0x%x device:%d '%s' (x:%.2f, y:%.2f, delta:%f, %f)",
                           (unsigned int) stage_x11->xwin,
+                          event->scroll.device->id,
                           event->scroll.device->device_name,
                           event->scroll.x,
                           event->scroll.y,
@@ -1022,9 +1035,6 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
                                                      &xev->buttons);
 
         clutter_event_set_source_device (event, source_device);
-
-        device = g_hash_table_lookup (manager_xi2->devices_by_id,
-                                      GINT_TO_POINTER (xev->deviceid));
         clutter_event_set_device (event, device);
 
         event->motion.axes = translate_axes (event->motion.device,
@@ -1041,8 +1051,9 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
           _clutter_event_set_pointer_emulated (event, TRUE);
 #endif /* HAVE_XINPUT_2_2 */
 
-        CLUTTER_NOTE (EVENT, "motion: win:0x%x device:%s (x:%.2f, y:%.2f, axes:%s)",
+        CLUTTER_NOTE (EVENT, "motion: win:0x%x device:%d '%s' (x:%.2f, y:%.2f, axes:%s)",
                       (unsigned int) stage_x11->xwin,
+                      event->motion.device->id,
                       event->motion.device->device_name,
                       event->motion.x,
                       event->motion.y,
@@ -1098,9 +1109,10 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
         if (xev->flags & XITouchEmulatingPointer)
           _clutter_event_set_pointer_emulated (event, TRUE);
 
-        CLUTTER_NOTE (EVENT, "touch %s: win:0x%x device:%s (seq:%d, x:%.2f, y:%.2f, axes:%s)",
+        CLUTTER_NOTE (EVENT, "touch %s: win:0x%x device:%d '%s' (seq:%d, x:%.2f, y:%.2f, axes:%s)",
                       event->type == CLUTTER_TOUCH_BEGIN ? "begin" : "end",
                       (unsigned int) stage_x11->xwin,
+                      event->touch.device->id,
                       event->touch.device->device_name,
                       GPOINTER_TO_UINT (event->touch.sequence),
                       event->touch.x,
@@ -1145,8 +1157,9 @@ clutter_device_manager_xi2_translate_event (ClutterEventTranslator *translator,
         if (xev->flags & XITouchEmulatingPointer)
           _clutter_event_set_pointer_emulated (event, TRUE);
 
-        CLUTTER_NOTE (EVENT, "touch update: win:0x%x device:%s (seq:%d, x:%.2f, y:%.2f, axes:%s)",
+        CLUTTER_NOTE (EVENT, "touch update: win:0x%x device:%d '%s' (seq:%d, x:%.2f, y:%.2f, axes:%s)",
                       (unsigned int) stage_x11->xwin,
+                      event->touch.device->id,
                       event->touch.device->device_name,
                       GPOINTER_TO_UINT (event->touch.sequence),
                       event->touch.x,
